<?php

namespace Drupal\sessions;

use Drupal\Core\TempStore\PrivateTempStore as CorePrivateTempStore;
use Drupal\Core\TempStore\TempStoreException;
use function method_exists;
use const REQUEST_TIME;

/**
 * {@inheritdoc}
 */
class PrivateTempStore extends CorePrivateTempStore {

  /**
   * {@inheritdoc}
   */
  public function get($key) {
    $key = $this->createkey($key);
    if (($object = $this->storage->get($key)) && ($object->owner == $this->getOwner())) {
      if (!empty($object->expire) && method_exists($object->data, 'setExpirationDate')) {
        $object->data->setExpirationDate($object->expire);
      }
      return $object->data;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function set($key, $value, $sess_id = NULL) {
    if (!$sess_id) {
      $key = $this->createkey($key);
      $owner = $this->getOwner();
    }
    else {
      $key = $sess_id . ':' . $key;
      $owner = $sess_id;
    }
    if (!$this->lockBackend->acquire($key)) {
      $this->lockBackend->wait($key);
      if (!$this->lockBackend->acquire($key)) {
        throw new TempStoreException("Couldn't acquire lock to update item '$key' in '{$this->storage->getCollectionName()}' temporary storage.");
      }
    }

    $value = (object) [
      'owner' => $owner,
      'data' => $value,
      'updated' => (int) $this->requestStack->getMasterRequest()->server->get('REQUEST_TIME'),
      'expire' => REQUEST_TIME + $this->expire,
    ];
    $this->storage->setWithExpire($key, $value, $this->expire);
    $this->lockBackend->release($key);
  }

  /**
   * Set tempstore expiration time.
   *
   * @param int $expire
   *   Expiration time.
   */
  public function setExpire(int $expire) {
    $this->expire = $expire;
  }

}
